@@ -3,8 +3,8 @@
 # Makefile for Java API Compliance Checker
 # Install/remove the tool for GNU/Linux, FreeBSD and Mac OS X
 #
-# Copyright (C) 2011 Institute for System Programming, RAS
-# Copyright (C) 2011-2015 Andrey Ponomarenko's ABI Laboratory
+# Copyright (C) 2011 Russian Linux Verification Center
+# Copyright (C) 2011-2012 ROSA Lab
 #
 # Written by Andrey Ponomarenko
 #
@@ -24,7 +24,6 @@
 use Getopt::Long;
 Getopt::Long::Configure ("posix_default", "no_ignore_case");
 use File::Path qw(mkpath rmtree);
-use File::Spec qw(catfile file_name_is_absolute);
 use File::Copy qw(copy);
 use File::Basename qw(dirname);
 use Cwd qw(abs_path);
@@ -43,19 +42,23 @@ DESCRIPTION:
   Install $TOOL_SNAME command and private modules.
 
 USAGE:
-  sudo perl $0 -install -prefix /usr
-  sudo perl $0 -remove -prefix /usr
+  sudo perl $0 -install -prefix=/usr
+  sudo perl $0 -update -prefix=/usr
+  sudo perl $0 -remove -prefix=/usr
 
 OPTIONS:
   -h|-help
       Print this help.
 
   --prefix=PREFIX
-      Install files in PREFIX [/usr].
+      Install files in PREFIX [/usr/local].
 
   -install
       Command to install the tool.
 
+  -update
+      Command to update existing installation.
+
   -remove
       Command to remove the tool.
 
@@ -73,13 +76,14 @@ if(not @ARGV)
     exit(0);
 }
 
-my ($PREFIX, $DESTDIR, $Help, $Install, $Remove);
+my ($PREFIX, $DESTDIR, $Help, $Install, $Update, $Remove);
 
 GetOptions(
     "h|help!" => \$Help,
     "prefix=s" => \$PREFIX,
     "destdir=s" => \$DESTDIR,
     "install!" => \$Install,
+    "update!" => \$Update,
     "remove!" => \$Remove
 ) or exit(1);
 
@@ -90,25 +94,23 @@ sub scenario()
         print $HELP_MSG;
         exit(0);
     }
-    if(not $Install and not $Remove)
+    if($Config{"osname"}!~/linux|freebsd|openbsd|netbsd|macos|darwin|rhapsody/)
     {
-        print STDERR "ERROR: command is not selected (-install or -remove)\n";
+        print STDERR "The tool is ready-to-use without the need to install.\n";
+        print STDERR "This Makefile is for GNU/Linux, FreeBSD and Mac OS X.\n";
         exit(1);
     }
-    
-    if($Install)
-    { # remove old version first
-        $Remove = 1;
+    if(not $Install and not $Update and not $Remove)
+    {
+        print STDERR "ERROR: command is not selected (-install, -update or -remove)\n";
+        exit(1);
     }
-    
     if($PREFIX ne "/") {
         $PREFIX=~s/[\/]+\Z//g;
     }
     if(not $PREFIX)
     { # default prefix
-        if($Config{"osname"}!~/win/i) {
-            $PREFIX = "/usr";
-        }
+        $PREFIX = "/usr/local";
     }
     if(my $Var = $ENV{"DESTDIR"})
     {
@@ -120,7 +122,7 @@ sub scenario()
         if($DESTDIR ne "/") {
             $DESTDIR=~s/[\/]+\Z//g;
         }
-        if(not isAbs($DESTDIR))
+        if($DESTDIR!~/\A\//)
         {
             print STDERR "ERROR: destdir is not absolute path\n";
             exit(1);
@@ -139,7 +141,7 @@ sub scenario()
     }
     else
     {
-        if(not isAbs($PREFIX))
+        if($PREFIX!~/\A\//)
         {
             print STDERR "ERROR: prefix is not absolute path\n";
             exit(1);
@@ -154,56 +156,52 @@ sub scenario()
     print "INSTALL PREFIX: $PREFIX\n";
     
     # paths
-    my $EXE_PATH = catFile($PREFIX, "bin");
-    my $MODULES_PATH = catFile($PREFIX, "share", $TOOL_SNAME);
-    my $REL_PATH = catFile("..", "share", $TOOL_SNAME);
-    my $TOOL_PATH = catFile($EXE_PATH, $TOOL_SNAME);
+    my $EXE_PATH = "$PREFIX/bin";
+    my $MODULES_PATH = "$PREFIX/share/$TOOL_SNAME";
+    my $REL_PATH = "../share/$TOOL_SNAME";
     
     if(not -w $PREFIX)
     {
         print STDERR "ERROR: you should be root\n";
         exit(1);
     }
-    if($Remove)
+    if($Remove or $Update)
     {
-        if(-e $EXE_PATH."/".$TOOL_SNAME)
-        { # remove executable
-            print "-- Removing $TOOL_PATH\n";
-            unlink($EXE_PATH."/".$TOOL_SNAME);
-        }
-        elsif(not $Install) {
-            print "The tool is not installed\n";
-        }
+        # remove executable
+        print "-- Removing $EXE_PATH/$TOOL_SNAME\n";
+        unlink($EXE_PATH."/".$TOOL_SNAME);
         
         if(-d $MODULES_PATH)
         { # remove modules
             print "-- Removing $MODULES_PATH\n";
             rmtree($MODULES_PATH);
         }
-        elsif(not $Install) {
-            print "The modules of the tool are not installed\n";
-        }
     }
-    if($Install)
+    if($Install or $Update)
     {
+        if(-e $EXE_PATH."/".$TOOL_SNAME or -e $MODULES_PATH)
+        { # check installed
+            if(not $Remove)
+            {
+                print STDERR "ERROR: you should remove old version first (`sudo perl $0 -remove --prefix=$PREFIX`)\n";
+                exit(1);
+            }
+        }
+        
         # configure
         my $Content = readFile($ARCHIVE_DIR."/".$TOOL_SNAME.".pl");
         if($DESTDIR) { # relative path
-            $Content=~s/MODULES_INSTALL_PATH/$REL_PATH/;
+            $Content=~s/ACC_MODULES_INSTALL_PATH/$REL_PATH/;
         }
         else { # absolute path
-            $Content=~s/MODULES_INSTALL_PATH/$MODULES_PATH/;
+            $Content=~s/ACC_MODULES_INSTALL_PATH/$MODULES_PATH/;
         }
         
         # copy executable
-        print "-- Installing $TOOL_PATH\n";
+        print "-- Installing $EXE_PATH/$TOOL_SNAME\n";
         mkpath($EXE_PATH);
         writeFile($EXE_PATH."/".$TOOL_SNAME, $Content);
-        chmod(0755, $EXE_PATH."/".$TOOL_SNAME);
-        
-        if($Config{"osname"}=~/win/i) {
-            writeFile($EXE_PATH."/".$TOOL_SNAME.".cmd", "\@perl \"$TOOL_PATH\" \%*");
-        }
+        chmod(0775, $EXE_PATH."/".$TOOL_SNAME);
         
         # copy modules
         if(-d $ARCHIVE_DIR."/modules")
@@ -214,32 +212,13 @@ sub scenario()
         }
         
         # check PATH
-        my $Warn = "WARNING: your PATH variable doesn't include \'$EXE_PATH\'\n";
-        
-        if($Config{"osname"}=~/win/i)
-        {
-            if($ENV{"PATH"}!~/(\A|[:;])\Q$EXE_PATH\E[\/\\]?(\Z|[:;])/i) { 
-                print $Warn;
-            }
-        }
-        else
-        {
-            if($ENV{"PATH"}!~/(\A|[:;])\Q$EXE_PATH\E[\/\\]?(\Z|[:;])/) { 
-                print $Warn;
-            }
+        if($ENV{"PATH"}!~/(\A|:)\Q$EXE_PATH\E(\Z|:)/) {
+            print "WARNING: your PATH variable doesn't include \'$EXE_PATH\'\n";
         }
     }
     exit(0);
 }
 
-sub catFile(@) {
-    return File::Spec->catfile(@_);
-}
-
-sub isAbs($) {
-    return File::Spec->file_name_is_absolute($_[0]);
-}
-
 sub copyDir($$)
 {
     my ($From, $To) = @_;
